@@ -3,7 +3,7 @@
 """
 Directory : mistool
 Name      : logTestUse
-Version   : 2013.04
+Version   : 2013.09
 Author    : Christophe BAL
 Mail      : projetmbc@gmail.com
 
@@ -355,15 +355,12 @@ This function uses the following variables.
     tests. By default, ``sort = lambda x: x``.
     """
 # Let's look for the good Python files.
-    pathTestFiles = [
-        onePath
-        for onePath in os_use.nextFile(
-            main  = dir,
-            ext   = {'keep': ["py"]},
-            depth = depth
-        )
-        if os_use.name(onePath).startswith('test_')
-    ]
+    pathTestFiles = os_use.listFile(
+        main   = dir,
+        ext    = "py",
+        prefix = "test_",
+        depth  = depth
+    )
 
 # Let's sort the paths.
     pathTestFiles.sort(key = sort)
