@@ -3,7 +3,7 @@
 """
 Directory : mistool
 Name      : osUse
-Version   : 2013.06
+Version   : 2013.09
 Author    : Christophe BAL
 Mail      : projetmbc@gmail.com
 
@@ -528,39 +528,82 @@ be inspected or not.
             relativeDepth(main = main, sub = sub) + 1 <= depth
         )
 
-def __toKeepToDiscard(extVar):
+def __isFileGood(
+    path,
+    ext,
+    prefix
+):
     """
 -----------------
 Small description
 -----------------
 
-This small function is used by ``nextFile`` so as to produce two dictionaries
-``extToKeep`` and ``extToDiscard`` from the variable ``ext``.
+This small function is used by ``nextFile`` so as to know if the file found
+matches the search criteria about extensions and prefixes.
     """
-    if extVar == None:
-        extVar = {'keep': "..."}
+    path = name(path)
 
-    elif isinstance(extVar, str) \
-    or isinstance(extVar, list):
-        extVar = {'keep': extVar}
+    toKeep = True
 
-    extToKeep = extVar.get('keep', [])
+    if ext['discard'] and hasExtIn(path, ext['discard']):
+        toKeep = False
 
-    if isinstance(extToKeep, str):
-        extToKeep = [extToKeep]
+    if toKeep:
+        if ext['keep'] and not hasExtIn(path, ext['keep']):
+            toKeep = False
 
-    extToDiscard = extVar.get('discard', [])
+    if toKeep:
+        for onePrefix in prefix['discard']:
+            if path.startswith(onePrefix):
+                toKeep = False
+                break
 
-    if isinstance(extToDiscard, str):
-        extToDiscard = [extToDiscard]
+    if toKeep and prefix['keep']:
+        toKeep = False
 
-    return extToKeep, extToDiscard
+        for onePrefix in prefix['keep']:
+            if path.startswith(onePrefix):
+                toKeep = True
+                break
+
+    return toKeep
+
+def __toKeepToDiscard(extOrPref):
+    """
+-----------------
+Small description
+-----------------
+
+This small function is used by ``nextFile`` so as to produce one normalized
+dictionary of the variables ``ext`` and ``prefix``.
+    """
+    if extOrPref == None:
+        extOrPref = {'keep': "..."}
+
+    elif isinstance(extOrPref, str) \
+    or isinstance(extOrPref, list):
+        extOrPref = {'keep': extOrPref}
+
+    toKeep = extOrPref.get('keep', [])
+
+    if isinstance(toKeep, str):
+        toKeep = [toKeep]
+
+    toDiscard = extOrPref.get('discard', [])
+
+    if isinstance(toDiscard, str):
+        toDiscard = [toDiscard]
+
+    return {
+        'keep'   : toKeep,
+        'discard': toDiscard
+    }
 
 def nextFile(
     main,
     ext     = {},
+    prefix  = {},
     depth   = 0,
-    sub     = {},
     keepDir = None
 ):
     """
@@ -568,7 +611,7 @@ def nextFile(
 Small description
 -----------------
 
-This function is an iterator that sends path of files in a directory withe the
+This function is an iterator that sends path of files in a directory with the
 possibility to use some criteria of research.
 
 
@@ -653,6 +696,29 @@ info::
     presentation of the arguments.
 
 
+It is also possible to use criteria of search regarding to the prefixes of the
+names of the files. For example, the following code will only return the path of
+the file which name starts with "latexTest".
+
+python::
+    from mistool import os_use
+
+    for onePath in os_use.nextFile(
+        main   = "/Users/mistool",
+        prefix = "latex",
+        depth  = -1
+    ):
+        print(onePath)
+
+
+The preceding code will product the following output in a terminal.
+
+terminal::
+    /Users/mistool/debug/debug_latex_use/latexTest.pdf
+    /Users/mistool/debug/debug_latex_use/latexTest.tex
+    /Users/mistool/debug/debug_latex_use/latexTestBuilder.py
+
+
 -------------
 The arguments
 -------------
@@ -679,72 +745,22 @@ This function uses the following variables.
             }
 
 
-        The values giving the extensions can be either a single string, or a list of strings.
+        The values giving the extensions can be either a single string, or a list
+        of strings.
 
     By default, ``ext`` is an empty dictionary. This asks to send every kind of
     files.
 
-    3) ``depth`` is the maximal depth for the research. The very special value
+    3) ``prefix`` can be used in the same way that `èxt``must be used. This
+    variable indicates files to keep or discard regarding to the prefix of their
+    name.
+
+    4) ``depth`` is the maximal depth for the research. The very special value
     ``(-1)`` indicates that there is no maximum.
 
     The default value is ``0`` which asks to only look for in the direct content
     of the main directory to analyse.
 
-    4) Let see with one example how to define the dictionary value of the
-    variable ``sub``. We assume that the main directory path::``source_dev`` has
-    the following simple structure with two sub directories.
-
-    dir::
-        + source_dev
-            + change_log
-            [...]
-            + debug
-            [...]
-
-    Regarding to the main directory, the two sub directories have the relative
-    paths path::``/change_log`` and path::``/debug`` respectively.
-
-    Let also suppose that we only want to keep files having the extension
-    path::``txt`` in the sub directory path::``change_log``, and that we don't
-    want to look for files in teh directories contained in  the sub directory
-    path::``debug``. Here is the value to give to the variable ``sub`` so as to
-    achieve this. Here the research specifications for the sub directories erase
-    the ones for the main directory.
-
-    python::
-        {
-            'change_log': {'keep': "txt"},
-            'debug'     : {'depth': 0}
-        }
-
-    In the preceding dictionary, the keys are the relative path of the sub
-    directory with or without the leading slash path::``/``.
-
-    You can combine the use of the keys ``'depth'`` and ``'keep'`` for the same
-    sub directory, and it is also possible to indicate extensions to discard
-    with the key ``'discard'``.
-
-    We finish with one last example. Let suppose here that in the main directory
-    we are only looking for files with the extension path::``py``, and that we
-    want to look for this kind of files and also the ones having the extension
-    path::``txt`` in the sub directory path::``change_log``. For such a
-    situation, it suffises to use the key ``'inherit'`` in the dictionary that
-    defines the extensions to keep or to discard. Here is the value of the
-    variable ``sub`` to use.
-
-    python::
-        {
-            'change_log': {
-                'inherit': True ,
-                'keep'   : "txt"
-            }
-        }
-
-    The value associated to the key ``'inherit'`` can be a boolean argument, but
-    you can also use couple of boolean values like ``(False, True)`` which asks
-    to only inherit the extensions for the files to discard. By default, the key
-    ``'inherit'`` is associated to ``False``.
-
     5) The variable ``keepDir`` has been added so as to be used by the class
     ``TreeDir``, but you can play with it if you want. The possible values are
     the following ones.
@@ -779,92 +795,37 @@ This function uses the following variables.
             "\n\t+ {0}".format(main)
         )
 
-# Is there some specific researches regarding to some sub directories ?
-    listOfsubPath    = []
-    subRelativePaths = {}
+# Extensions and prefixes
+    ext    = __toKeepToDiscard(ext)
+    prefix = __toKeepToDiscard(prefix)
 
-    for oneRelativePath in sub:
-        realPath = oneRelativePath
-
-        if not realPath.startswith(SEP):
-            realPath = SEP + realPath
-
-        realPath = main + realPath
-
-        if not isDir(realPath):
-            raise OsUseError(
-                "The following path does not point to one existing "
-                "sub directory.\n\t+ {0}".format(realPath)
-            )
-
-        listOfsubPath.append(realPath)
-        subRelativePaths[realPath] = oneRelativePath
-
-    extToKeep, extToDiscard = __toKeepToDiscard(ext)
 
+# It's time to walk in the directory...
     for root, dirs, files in os.walk(main):
-### CASE OF ONE SUB DIRECTORY WITH SPECIAL CRITERIONS ###
-        if root in listOfsubPath:
-            subResearch = sub[subRelativePaths[root]]
-
-            inherit = subResearch.get('inherit', False)
-
-            subExtToKeep, subExtToDiscard = __toKeepToDiscard(subResearch)
-
-            subDepth = subResearch.get('depth', 0)
-
-# Inherit or not inherit ? That is the question...
-            if inherit:
-                subExtToKeep    += extToKeep
-                subExtToDiscard += extToDiscard
-                subDepth        = depth
-
-# Let's look for the files in the sub directory...
-            for oneFileInsub in nextFile(
-                main = root,
-                ext  = {
-                    'keep'   : subExtToKeep,
-                    'discard': subExtToDiscard
-                },
-                depth   = subDepth,
-                keepDir = keepDir
-            ):
-                yield oneFileInsub
-
-### CASE OF ONE SUB DIRECTORY WITHOUT SPECIAL CRITERIONS ###
-        elif __isSubDepthGood(
+        if __isSubDepthGood(
             main  = main,
             sub   = root,
             depth = depth
         ):
+# The following boolean are used for the directory views !
             noBadFileFound  = True
             noGoodFileFound = True
 
             for oneFile in files:
                 path = root + SEP + oneFile
 
-                if not hasExtIn(path, extToDiscard):
-                    if extToKeep:
 # Looking for good files
-                        if hasExtIn(path, extToKeep):
+                if __isFileGood(path, ext, prefix):
 # Directory of the first good file found to display ?
-                            if noGoodFileFound and dirMustBeKept:
-                                noGoodFileFound = False
-                                yield root
+                    if noGoodFileFound and dirMustBeKept:
+                        noGoodFileFound = False
+                        yield root
 
-                            yield path
+                    yield path
 
 # One bad file found
-                        elif noBadFileFound:
-                            noBadFileFound = False
-
-# Looking for any file
-                    else:
-                        if noGoodFileFound and dirMustBeKept:
-                            noGoodFileFound = False
-                            yield root
-
-                        yield path
+                elif noBadFileFound:
+                    noBadFileFound = False
 
 # Directory without any good file
             if noGoodFileFound and keepDir in ["all", "All"]:
@@ -894,6 +855,121 @@ list returned by ``listFile``.
     return [p for p in nextFile(*args, **kwargs)]
 
 
+# ----------------------- #
+# -- ABOUT DIRECTORIES -- #
+# ----------------------- #
+
+def nextDir(
+    main,
+    depth = 0
+):
+    """"
+-----------------
+Small description
+-----------------
+
+This function is an iterator that sends path of directories contained in a
+directory.
+
+
+Suppose for example that we have the following directory structure.
+
+directory::
+    + mistool
+        * __init__.py
+        * description.rst
+        * latex_use.py
+        * os_use.py
+        * ...
+        + change_log
+            + 2012
+                * 07.pdf
+                * 08.pdf
+                * 09.pdf
+        + debug
+            * debug_latex_use.py
+            * debug_os_use.py
+            + debug_latex_use
+                * latexTest.pdf
+                * latexTest.tex
+                * latexTestBuilder.py
+                * ...
+        + toUse
+            + latex
+                * ...
+
+
+Let's consider the following code.
+
+python::
+    from mistool import os_use
+
+    for onePath in os_use.nextFile(
+        main  = "/Users/mistool",
+        depth = -1
+    ):
+        print(onePath)
+
+
+If we launch the preceding code in a terminal, then we obtain the following
+outputs.
+
+terminal::
+    /Users/mistool/change_log
+    /Users/mistool/change_log/2012
+    /Users/mistool/debug
+    /Users/mistool/debug/debug_latex_use
+    /Users/mistool/toUse
+    /Users/mistool/toUse/latex
+
+
+-------------
+The arguments
+-------------
+
+This function uses the following variables.
+
+    1) ``main`` is simply the path of the main directory to analyse.
+
+    2) ``depth`` is the maximal depth for the research. The very special value
+    ``(-1)`` indicates that there is no maximum.
+
+    The default value is ``0`` which asks to only look for in the direct content
+    of the main directory to analyse.
+    """
+    firstDir = True
+
+    for onePath in os_use.nextFile(
+        main    = main,
+        ext     = None,
+        depth   = -1,
+        keepDir = "all"
+    ):
+        if firstDir:
+            firstDir = False
+
+        else:
+            yield onePath
+
+def listDir(*args, **kwargs):
+    """
+-----------------
+Small description
+-----------------
+
+This function is similar to the function ``nextDir``, and it has the same
+variables that the ones of ``nextDir``, but instead of sending infos about the
+directories found one at a time, this function directly sends the whole list of
+the infos found.
+
+See the documentation of the function ``nextDir`` to have precisions about the
+available variables and the structure of each single info that will be in the
+list returned by ``listDir``.
+    """
+# The use of ``*args`` and ``**kwargs`` makes it very easy to implement the fact
+# that ``listFile`` and ``nextFile`` have the same variables.
+    return [p for p in nextDir(*args, **kwargs)]
+
 class DirView:
     """
 -----------------
@@ -979,8 +1055,8 @@ This class uses the following variables.
         self,
         main,
         ext     = {},
+        prefix  = {},
         depth   = 0,
-        sub     = {},
         keepDir = "minimal",
         output  = "all",
         seeMain = True
@@ -993,8 +1069,8 @@ This class uses the following variables.
 
         self.main    = main
         self.ext     = ext
+        self.prefix  = prefix
         self.depth   = depth
-        self.sub     = sub
         self.keepDir = keepDir
         self.output  = output
         self.seeMain = seeMain
@@ -1022,8 +1098,8 @@ python::
         for onePath in nextFile(
             main    = self.main,
             ext     = self.ext,
+            prefix  = self.prefix,
             depth   = self.depth,
-            sub     = self.sub,
             keepDir = self.keepDir
         ):
             if onePath == self.main:
