@@ -3,7 +3,7 @@
 """
 Directory : mistool
 Name      : stringUse
-Version   : 2013.06
+Version   : 2013.07
 Author    : Christophe BAL
 Mail      : projetmbc@gmail.com
 
@@ -342,7 +342,7 @@ function ``replace`` with the attribut ``self.directReplacement``).
 
 # --- START-ASCII ---#
 
-# The following dictionary was automatically built by one Python script.
+# The two following constants were automatically built by one Python script.
 
 REPLACEMENT_ASCII = {
     '¡': '', '¿': '', 'œ': 'oe', 'à': 'a', 'â': 'a', 'ä': 'a', 'ç': 'c',
@@ -351,9 +351,22 @@ REPLACEMENT_ASCII = {
     'ÿ': 'y'
 }
 
+ASCII_CHAR = {
+    ' ', '$', '(', ',', '0', '4', '8', '<', '@', 'D', 'H', 'L', 'P', 'T',
+    'X', '\\', '`', 'd', 'h', 'l', 'p', 't', 'x', '|', '#', "'", '+', '/',
+    '3', '7', ';', '?', 'C', 'G', 'K', 'O', 'S', 'W', '[', '_', 'c', 'g',
+    'k', 'o', 's', 'w', '{', '"', '&', '*', '.', '2', '6', ':', '>', 'B',
+    'F', 'J', 'N', 'R', 'V', 'Z', '^', 'b', 'f', 'j', 'n', 'r', 'v', 'z',
+    '~', '!', '%', ')', '-', '1', '5', '9', '=', 'A', 'E', 'I', 'M', 'Q',
+    'U', 'Y', ']', 'a', 'e', 'i', 'm', 'q', 'u', 'y', '}'
+}
+
 # --- END-ASCII ---#
 
-def ascii(text):
+def ascii(
+    text,
+    strict = True
+):
     """
 -----------------
 Small description
@@ -368,6 +381,7 @@ python::
         string_use.ascii("¡Viva España!")
     )
 
+
 You can change the constant ``REPLACEMENT_ASCII`` like in the following lines so as to simply obtain ``Viva Espana`` rather than ``Viva Espana!``.
 
 python::
@@ -387,8 +401,8 @@ It's easy to improve the special characters managed by default.
     ``string_use.REPLACEMENT_ASCII``.
 
     2) If you think that your tuning is general enough, just send to the author
-    of this module a text file containing the special characters with their
-    ¨ascii translation of the following kind (don't use ``code::``).
+    of this module a text file of the following kind wich contains the special
+    characters with their ¨ascii translation (don't use ``code::``).
 
     code::
         a::
@@ -404,12 +418,46 @@ It's easy to improve the special characters managed by default.
         1) The characters ``à``, ``ä`` and ``â`` must be translated to ``a``.
 
         2) The characters ``¡`` and ``¿`` must be removed.
+
+
+-------------
+The arguments
+-------------
+
+This function uses the following variables.
+
+    1) ``text`` is simply the text to be translated.
+
+    2) ``strict`` is an optional argument to use if you don't want the function to raise an error when the translation has been partial.
+
+    By default, ``strict = True`` indicates to raise an error.
     """
-    return replace(
+    asciiText = replace(
         text        = text,
         replacement = REPLACEMENT_ASCII
     )
 
+    if strict:
+        extraChar = set(asciiText) - ASCII_CHAR
+
+        if extraChar:
+            if len(extraChar) == 1:
+                plurial = ""
+
+            else:
+                plurial = "s"
+
+            extraChar = " , ".join([
+                "<< {0} >>".format(x) for x in extraChar
+            ])
+
+            raise StringUseError(
+                "The following character{0} can't be translated : {1} ."\
+                    .format(plurial, extraChar)
+            )
+
+    return asciiText
+
 
 # ----------------- #
 # -- LOOKING FOR -- #
@@ -461,8 +509,9 @@ given. If the prefix is empty, the matching words are all the words defining the
 auto completion.
 
 This search indeed uses a special "magical" dictionary which is stored in the
-attribut ``dict``. Here ``myAC.dict`` is equal to the dictionary above where the
-lists of integers correspond to the good indexes in the list of words.
+attribut ``dict``. With the preceding example, ``myAC.dict`` is equal to the
+dictionary above where the lists of integers correspond to the good indexes in
+the ordered list of words.
 
 python::
     {
@@ -537,9 +586,9 @@ The instanciation of this class uses the following variables.
     infoo:
         `3` seems to be a good value of ``depth`` for ¨gui application.
 
-    3) ``dictAsso`` is a "magical" dictionary that eases the auto completion.
-    This dictionary is build by the method ``build`` from the list of words tio
-    be used for the auto completions.
+    3) ``dict`` is a "magical" dictionary that eases the auto completion. This
+    dictionary is build by the method ``build`` from the list of words to be
+    used for the auto completions.
     """
 
     def __init__(
@@ -1332,7 +1381,8 @@ def __addNewWord(
     line,
     newLines,
     width,
-    indentation
+    indent,
+    lenIndent
 ):
     """
 -----------------
@@ -1360,62 +1410,78 @@ This function uses the following variables.
 
     4) ``width`` is the width expected for the lines.
 
-    5) ``indentation`` is the indentation of the current line analyzed.
-    """
-    lenWord        = len(word)
-    lenIndentation = len(indentation)
+    5) ``indent`` is the indentation of the current line analyzed.
 
-    if len(line) + lenWord > width:
+    6) ``lenIndent`` is the number of spaces of the indentation of the current
+    line analyzed.
+    """
+    lenWord = len(word)
+
+# The new word is too big.
+    if len(line) + lenWord >= width:
         newLines.append(line)
 
-        if lenWord + lenIndentation > width:
-            newLines.append(indentation + word)
+        if lenWord + lenIndent > width:
+            newLines.append(indent + word)
             line = ''
 
         else:
-            line = indentation + word
+            line = indent + word
 
+# The new word is small.
     elif word:
         if line:
             line += ' '
 
         else:
-            line = indentation
+            line = indent
 
         line += word
 
     return (line, newLines)
 
-def __indentation(text):
+def __indent(
+    text,
+    spaceTab
+):
     """
 -----------------
 Small description
 -----------------
 
 This function, which is used by the function ``cut``, simply returns the leading
-spaces and tabulations of one text.
+spaces and tabulations of one text. Each tabulation is converted to spaces using
+the variable ``spaceTab``.
 
 
 -------------
 The arguments
 -------------
 
-This function has only one variable ``text`` which is the text to analyse.
+This function uses the following arguments.
+
+    1) ``text`` is the text to analyse.
+
+    2) ``spaceTab`` indicates the string of spaces which replaces each
+    tabulation.
     """
-    indentation = ''
+    indent = ''
 
     if text.strip():
         i = 0
 
         while text[i] in " \t":
-            indentation += text[i]
+            indent += text[i]
             i+=1
 
-    return indentation
+    indent = indent.replace('\t', spaceTab)
+
+    return indent
 
 def cut(
     text,
-    width = 80
+    width    = 80,
+    spaceTab = ' '*4
 ):
     """
 -----------------
@@ -1423,92 +1489,109 @@ Small description
 -----------------
 
 This function tries to cut lines of a text so as they have less than `81`
-characters.
+characters (by default but you can change this number).
 
-Multiple following spaces are rendered as one single space, and the final text
-is stripped so as to start and end with none empty lines.
+Multiple following spaces are rendered as only one single space, and the final
+text is stripped so as to start and end with none empty lines.
 
-One last thing, when lines are cut their indentations are respected.
+One last thing, when lines are cut their indentations are respected but each
+leading tabulation will be replaced by spaces (see the description of the
+variable ``spaceTab``).
 
 
 -------------
 The arguments
 -------------
 
-There are two arguments.
+This function uses two arguments.
 
     1) ``text`` is simply the text to cut.
 
-    2) ``width`` is an integer that gives the maximum of characters wanted to be
-    in one line. Its default value is `80`.
+    2) ``width`` is an optional integer that gives the maximum of characters
+    wanted to be in one line.
+
+    By default, ``width = 80``.
+
+    3) ``spaceTab`` indicates the string of spaces which replaces each
+    tabulation.
     """
     if not isinstance(width, int) or width <= 0:
         raise StringUseError("<< width >> must be a positive integer.")
 
-    word        = ''
-    line        = ''
-    indentation = __indentation(text)
-    newLines    = []
+    newLines = []
 
-    for i in range(len(text)):
-        oneChar = text[i]
+# Let's work line by line.
+    for oldLine in text.split('\n'):
+        indent = __indent(
+            text     = oldLine,
+            spaceTab = spaceTab
+        )
 
-# One space
-        if oneChar == ' ':
-            line, newLines = __addNewWord(
-                word, line, newLines,
-                width, indentation
-            )
+        lenIndent = len(indent)
 
-            word = ''
+        oldLine = oldLine.strip()
+        word    = ''
+        line    = ''
 
-# One back return
-        elif oneChar == '\n':
-            line, newLines = __addNewWord(
-                word, line, newLines,
-                width, indentation
-            )
+# A not empty line.
+        if oldLine:
+            for oneChar in oldLine:
+# One space found...
+                if oneChar == ' ':
+                    line, newLines = __addNewWord(
+                        word,
+                        line, newLines,
+                        width,
+                        indent, lenIndent
+                    )
 
-            word = ''
-
-            indentation = __indentation(text[i+1:])
-
-            if line:
-                newLines.append(line)
-                line = ''
-
-            else:
-                newLines.append('')
+                    word = ''
 
 # The word becomes bigger...
+                else:
+                    word += oneChar
+
+# The end of the line
+            if word:
+                line, newLines = __addNewWord(
+                    word,
+                    line, newLines,
+                    width,
+                    indent, lenIndent
+                )
+
+                if line:
+                    newLines.append(line)
+                    line = ''
+
+                else:
+                    newLines.append('')
+
+# An empty line
         else:
-            word += oneChar
+            newLines.append('')
 
-# Let's finish the job !
-    line, newLines = __addNewWord(
-        word, line, newLines,
-        width, indentation
-    )
-
-    if line:
-        newLines.append(indentation + line)
-
-    return '\n'.join(newLines).strip()
+# All the job has been done !
+    return '\n'.join(newLines)
 
 
 # ------------------- #
 # -- DECORATE TEXT -- #
 # ------------------- #
 
-# The default frames
+# The default frames :
+#
 #    * ``C_FRAME``
 #    * ``C_PRETTY_FRAME``
+#
 #    * ``PYTHON_FRAME``
 #    * ``PYTHON_PRETTY_FRAME``
+#
 #    * ``PYBA_NB_PART_FRAME``
 #    * ``PYBA_NO_NB_PART_FRAME``
 #    * ``PYBA_NB_TITLE_FRAME``
 #    * ``PYBA_NO_NB_TITLE_FRAME``
+#
 #    * ``UNITTEST_FRAME``
 #    * ``UNITTEST_PROBLEM_FRAME``
 
@@ -1674,7 +1757,7 @@ This function uses the following variables.
 
 def frame(
     text,
-    format,
+    format = PYTHON_FRAME,
     center = True
 ):
     """
@@ -1682,6 +1765,9 @@ def frame(
 Small description
 -----------------
 
+?????
+
+
 This function makes it possible to put one text into one frame materialized by
 ASCII characters. This can be usefull for console outputs or for pretty comments
 for listings like the following python comment.
